
#ifndef __H_TEST_MACROS__
#define __H_TEST_MACROS__

#include <stdio.h>

#ifndef RESET
/* Colour definitions for console prints */
#define RESET   "\033[0m"
#define BLACK   "\033[30m"      /* Black */
#define RED     "\033[31m"      /* Red */
#define GREEN   "\033[32m"      /* Green */
#define YELLOW  "\033[33m"      /* Yellow */
#define BLUE    "\033[34m"      /* Blue */
#define MAGENTA "\033[35m"      /* Magenta */
#define CYAN    "\033[36m"      /* Cyan */
#define WHITE   "\033[37m"      /* White */
#define BOLDBLACK   "\033[1m\033[30m"      /* Bold Black */
#define BOLDRED     "\033[1m\033[31m"      /* Bold Red */
#define BOLDGREEN   "\033[1m\033[32m"      /* Bold Green */
#define BOLDYELLOW  "\033[1m\033[33m"      /* Bold Yellow */
#define BOLDBLUE    "\033[1m\033[34m"      /* Bold Blue */
#define BOLDMAGENTA "\033[1m\033[35m"      /* Bold Magenta */
#define BOLDCYAN    "\033[1m\033[36m"      /* Bold Cyan */
#define BOLDWHITE   "\033[1m\033[37m"      /* Bold White */
#endif

#define setup_unit()            \
  unsigned int unit_retval = 0;

#define end_unit()    \
  return unit_retval;

#define setup_suite()                   \
  unsigned int total_fails = 0;         \
  unsigned int test_suites_failed = 0;  \
  unsigned int tmp_val;                 \

#define end_suite()                                            \
  if (test_suites_failed > 0){                                 \
    fprintf (stderr, RED "[TEST SUITE FAILED]\n"               \
    " Number of failed suits: %d\n"                            \
    " Number of tests failed (total): %d\n" RESET,             \
    test_suites_failed, total_fails);                          \
  } else {                                                     \
    fprintf (stdout, GREEN "[TEST SUITE PASSED]\n" RESET);     \
  }                                                            \
  return total_fails + test_suites_failed;

#define test_fail(p, ...)\
  fprintf (stderr, RED "[FAILED][%s:%d, %s] " p " \n" RESET,   \
  __FILE__, __LINE__, __func__ , ##__VA_ARGS__)

#define test_pass(p, ...)                            \
   fprintf (stdout, GREEN "[PASS] " p " \n" RESET,   \
   ##__VA_ARGS__)

#define test_case(cond, p)          \
    if (cond) {                     \
      test_pass(p);                 \
    } else {                        \
      test_fail(p);                 \
      unit_retval++;                \
    }

#define test_unit(fun, name)                                     \
  tmp_val = fun;                                                 \
  if (tmp_val > 0) {                                             \
    total_fails =+ tmp_val;                                      \
    test_suites_failed++;                                        \
    fprintf (stderr, RED "[TEST UNIT FAIL] " name "\n" RESET);   \
  } else {                                                       \
    fprintf (stdout, GREEN "[TEST UNIT PASS] " name "\n" RESET); \
  }

#endif /* __H_TEST__ */
